/*********************************************************************
	DAWSON accelerator
*********************************************************************/
#include "em1.h"

Dawsacc(mno1,mno2,k0,forback,e01,e02,freq1,freq2,rgtlft1,rgtlft2,wc1x,
			ey1,ey2,ez1,ez2)
int mno1,mno2;
SCALAR k0,forback,e01,e02,freq1,freq2,rgtlft1,rgtlft2,wc1x,ey1,ey2,
		ez1,ez2;
{
  int i;
  SCALAR csp,kem1,kem2,ey1p,ey2p,ez1p,ez2p,ey1m,ey2m,ez1m,ez2m,
        veyr1,veyr2,vezr1,vezr2,v01,v02,qm1,wp1;
  csp = dx/dt;
  
  /* FORBACK defines colinear/antilinear wave beating*/
  /* 1.0 = colinear; -1.0 = antilinear*/
  kem1	= mno1*k0;
  kem2	= mno2*k0*forback;
  qm1	= qs[0]/ms[0];
  wp1	= qm1*sqrt(np[0]*ms[0]);

  if (circular)
    {
      /* Circular polarization scheme. */
      /* Program assumes frequencies input by user follow dispersion relation*/
      /* kem = freq/csp*(1.0-wp1^2/(freq*(freq -/+ wc1x)))^0.5 			*/
      /* where -/+ means - for right CP waves and + for left CP waves. 		*/
      /* Compute velocity perturbations and field contributions of waves. 	*/

      v01 = qm1*e01/(freq1 - rgtlft1*wc1x);
      v02 = qm1*e02/(freq2 - rgtlft2*wc1x);
      for (i=0; i<np[0]; i++)
	{
	  vy[0][i] += (v01*sin(kem1*pos[0][i]) + v02*sin(kem2*pos[0][i]))/sqrt(2.0);
	  vz[0][i] += (rgtlft1*v01*cos(kem1*pos[0][i]) +
		       rgtlft2*v02*cos(kem2*pos[0][i]))/sqrt(2.0);
	}
      ey1p	= e01/(2.0*sqrt(2.0)) * (1.0+csp*kem1/freq1);
      ey2p	= e02/(2.0*sqrt(2.0)) * (1.0+csp*kem2/freq2);
      ez1p	= -1.0*rgtlft1*ey1p;
      ez2p	= -1.0*rgtlft2*ey2p*forback;
      ey1m	= e01/(2.0*sqrt(2.0)) * (1.0-csp*kem1/freq1);
      ey2m	= e02/(2.0*sqrt(2.0)) * (1.0-csp*kem2/freq2);
      ez1m	= -1.0*rgtlft1*ey1m;
      ez2m	= -1.0*rgtlft2*ey2m*forback;
      for (i=0; i<ng; i++)
	{
	  eyr[i] = ey1p*cos(mno1*TWOPI*i/nc) + ey2p*cos(mno2*TWOPI*i/nc);
	  eyl[i] = ey1m*cos(mno1*TWOPI*i/nc) + ey2m*cos(mno2*TWOPI*i/nc);
	  ezr[i] = ez1p*sin(mno1*TWOPI*i/nc) + ez2p*sin(mno2*TWOPI*i/nc);
	  ezl[i] = ez1m*sin(mno1*TWOPI*i/nc) + ez2m*sin(mno2*TWOPI*i/nc);
	}
    }
  else
    {
      /* Linear polarization scheme. */
      /* User chooses orientation by altering amplitudes ey,ez appropriately.*/
      /* Computer finds frequencies with simple dispersion relation. */
      freq1 = sqrt((kem1*csp)*(kem1*csp) + wp1*wp1) + 1.0e-10;
      freq2 = sqrt((kem2*csp)*(kem2*csp) + wp1*wp1) + 1.0e-10;
      veyr1 = qm1*ey1/(freq1*csp);
      veyr2 = qm1*ey2/(freq2*csp);
      vezr1 = qm1*ez1/(freq1*csp);
      vezr2 = qm1*ez2/(freq2*csp);
      ey1p	= ey1*0.5*(1 + kem1*csp/freq1);
      ey2p	= ey2*0.5*(1 + kem2*csp/freq2)*forback;
      ez1p	= ez1*0.5*(1 + kem1*csp/freq1);
      ez2p	= ez2*0.5*(1 + kem2*csp/freq2);
      ey1m	= ey1*0.5*(1 - kem1*csp/freq1);
      ey2m	= ey2*0.5*(1 - kem2*csp/freq2)*forback;
      ez1m	= ez1*0.5*(1 - kem1*csp/freq1);
      ez2m	= ez2*0.5*(1 - kem2*csp/freq2);
      for (i=0; i<np[0]; i++)
	{
	  vy[0][i]	+= veyr1*cos(kem1*pos[0][i]) + veyr2*cos(kem2*pos[0][i]);
	  vz[0][i]	-= vezr1*sin(kem1*pos[0][i]) + vezr2*sin(kem2*pos[0][i]);
	}
      for (i=0; i<ng; i++)
	{
	  eyr[i] = ey1p*sin(mno1*TWOPI*i/nc) + ey2p*sin(mno2*TWOPI*i/nc);
	  eyl[i] = ey1m*sin(mno1*TWOPI*i/nc) + ey2m*sin(mno2*TWOPI*i/nc);
	  ezr[i] = ez1p*cos(mno1*TWOPI*i/nc) + ez2p*cos(mno2*TWOPI*i/nc);
	  ezl[i] = ez1m*cos(mno1*TWOPI*i/nc) + ez2m*cos(mno2*TWOPI*i/nc);
	}
    }
}
