/* em1.c */
/***************************************************************** 
**	EM1 - ELECTROMAGNETIC 1 DIMENSIONAL PLASMA SIMULATION
**
**
**	REVISION/PROGRAMMER/DATE
**	
**	1.0	 Original Fortran code written by A. Bruce Langdon, Livermore, 1972 and
**		 B. Cohen, M Mostrom, and D. Nicholson, UC Berkeley, 1973.
**
**	2.0      Conversion from FORTRAN to C by Edward H. Chao 1990
**                    using WinGraphics interface by John P. Verboncoeur and 
**                    Vahid Vahedi
**
**      3.0      Conversion from WinGraphics interface to XGRAFIX interface
**                    by Edward H. Chao 1991
**                    XGRAFIX interface/Vahid Vahedi and John Verboncoeur/
**
******************************************************************/

#define GLOBALORIGIN
#include "em1.h"
#include "xgrafix.h"
#undef GLOBALORIGIN


main(int argc,char * argv[]) 
{
  int i, msg=0;
  
  XGInit(argc, argv, &t);
  display_title();
  Start(argc,argv);
  InitWindows();			/*Initialize Windows*/

  History();
  
  /************************************/
  /* Start of loop to evolve in time  */
  XGStart();
  

}

void XGMainLoop() {
  int i;
    t += dt;
    
    for (i=0; i<nsp; i++)
      Mover(i);
    Fields();
    History();
}
	
/******************End of Main************************************/

display_title()
{
  printf("\n\nEM1 - Electromagnetic 1 Dimensional Code\n");
  printf("version 3.0\n");
  printf("(c) Copyright 1987-91 Regents of the University of California\n");
  printf("Plasma Theory and Simulation Group\n");
  printf("University of California - Berkeley\n\n");
}

/******************End of Display*********************************/

Start(int argc, char *argv[])
{
  char a_char[80];
  int i, j, mno1, mno2, mode;
  SCALAR wc1x,wc1z,ey1,ey2,ez1,ez2,forback,e01,e02,rgtlft1,rgtlft2,
  freq1,freq2,k0,eloss;
	
  /**********************************************************/
  /* Setting the initial parameters to their default values. */
	
  nsp	= 1;				/* Number of species */
  wc1x = wc1z = 0.0;			/*Cyclotron freq. of 1st species in B0x and B0z fields*/
  mode = 1;				/* # of the mode for position perturbations*/
  k0 = .5;				/* 2*PI/Length of system */
  nc = 32;				/* # of grid cells*/
  dt = 0.1;				/* Time step */
  dawsacc = False;			/* Chooses Dawson accelerator for launching 
					   self-consistent travelling waves */
  nkays = 8;				/* # of Fourier modes ( <= 8 )*/
  for (i=0; i<8; i++)
  kay[i] = i+1;				/* Integer array containing Fourier mode #'s */
  ey1 = ey2 = 0.0;			/* Peak Ey values of the 2 beat waves */
  ez1 = ez2 = 0.0;			/* Peak Ez values of the 2 beat waves */
  mno1 = mno2 = 0;			/* Beat wave mode #'s over K0 */
  forback = 1.0;			/* Alignment of beat waves: 1.0=colinear, -1.0 = antilinear */
  circular = False;			/* Chooses circularly polarized launched waves */
  e01 = e02 = 0.0;			/* Peak values of the 2 beat waves */
  rgtlft1 = rgtlft2 = 1.0;		/* Polarization of beat waves: 1.0=right, -1.0 = left */
  freq1 = freq2 = 0.1;			/* Freq. of the 2 beat waves */

  /***********************************************************/
	
  InputDeck = fopen(theInputFile,"r");
  
	
  if (argc > 1 && !InputDeck)
    {
      for (i=0; argv[1][i] != 0; i++) a_char[i] = argv[1][i];
      a_char[i++] = '.';		/*try to add .inp */
      a_char[i++] = 'i';
      a_char[i++] = 'n';
      a_char[i++] = 'p';
      a_char[i++] = 0;
      InputDeck = fopen(a_char,"r");
    }
  if (!InputDeck)
    {
      printf("\nCan't find input file %s\n",argv[2]);
      printf("\nCorrect syntax is: EM1 file.INP");
      printf("\nUsing Default values.\n");
    }
  /* read lines until we get to numbers */
	
  if (InputDeck)
    {
#ifndef XG_SCALAR_DOUBLE
        while (fscanf(InputDeck," %d %d %g %g %d",
#else
            while (fscanf(InputDeck," %d %d %lg %lg %d",
#endif
		    &nsp, &nc, &k0, &dt, &mode) < 5)
	fscanf(InputDeck, "%s", a_char);

#ifndef XG_SCALAR_DOUBLE
while (fscanf(InputDeck,"%d %d %g %g %g",
#else
    while (fscanf(InputDeck,"%d %d %lg %lg %lg",
#endif
		    &dawsacc, &circular, &forback, &wc1x, &wc1z) <5)
	fscanf(InputDeck, "%s", a_char);

#ifndef XG_SCALAR_DOUBLE
while (fscanf(InputDeck,"%d %d %g %g %g %g",
#else
    while (fscanf(InputDeck,"%d %d %lg %lg %lg %lg",
#endif
		    &mno1, &mno2, &e01, &e02, &freq1, &freq2) <6)
	fscanf(InputDeck, "%s", a_char);

#ifndef XG_SCALAR_DOUBLE
while (fscanf(InputDeck,"%g %g %g %g %g %g",
#else
    while (fscanf(InputDeck,"%lg %lg %lg %lg %lg %lg",
#endif
		    &rgtlft1, &rgtlft2, &ey1, &ey2, &ez1, &ez2) <6)
	fscanf(InputDeck, "%s", a_char);

      /*************************Check Input Values********************/
      if (nc > MAXG)
	{
	  printf("Number of grid cells cannot exceed MAXG\n");
	  exit(-1);
	}
      if (nc == 0)
	{
	  printf("Number of grid cells cannot equal 0!\n");
	  exit(1);
	}
      if (k0 == 0.0)
	{
	  printf("k0 cannot equal 0!\n");
	  exit(1);
	}
    }
  printf("nsp=%d\tnc=%d\tk0=%g\tdt=%g\tmode=%d\n",
	 nsp,nc,k0,dt,mode);

  printf("dawsacc=%d\tcircular=%d\tforback=%g\twc1x=%G\twc1z=%G\n",
	 dawsacc, circular, forback, wc1x, wc1z);

  for (i=0; i<8; i++)
    printf("kay[%d]=%d\t",i,kay[i]);

  ng = nc+1;
  length = TWOPI/k0;
  cgshl = length;
  dx = length/nc;
		
  /*******************************/
  /* Allocating space for arrays */
  x_array= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  for (i=0; i<ng; i++)	x_array[i]=dx*i;

  ms		= (SCALAR *)calloc(1,nsp*sizeof(SCALAR));
  qs		= (SCALAR *)calloc(1,nsp*sizeof(SCALAR));
  nm		= (SCALAR *)calloc(1,nsp*sizeof(SCALAR));
  rnmc2i          = (SCALAR *)calloc(1,nsp*sizeof(SCALAR));
  px		= (SCALAR *)calloc(1,nsp*sizeof(SCALAR));
  py		= (SCALAR *)calloc(1,nsp*sizeof(SCALAR));
  pz		= (SCALAR *)calloc(1,nsp*sizeof(SCALAR));
  tx		= (SCALAR *)calloc(1,nsp*sizeof(SCALAR));
  tz		= (SCALAR *)calloc(1,nsp*sizeof(SCALAR));

  eyl	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  eyr	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  ezl	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  ezr	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  jym	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  jyp	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  jzm	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  jzp	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  phi	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  rho	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  thermx  = (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  ex 	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  ey 	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  ez 	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  by 	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  bz 	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  vxosc	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  vyosc	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  vzosc	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  ek 	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));
  ksq 	= (SCALAR *)calloc(1,(ng+1)*sizeof(SCALAR));

  pos	= (SCALAR **)calloc(1,nsp*sizeof(SCALAR *));
  vx		= (SCALAR **)calloc(1,nsp*sizeof(SCALAR *));
  vy 	= (SCALAR **)calloc(1,nsp*sizeof(SCALAR *));
  vz 	= (SCALAR **)calloc(1,nsp*sizeof(SCALAR *));
  gamma_func	= (SCALAR **)calloc(1,nsp*sizeof(SCALAR *));

  for (i=0; i<nsp; i++)
    {
      pos[i]= (SCALAR *)calloc(1,MAXNP*sizeof(SCALAR));
      vx[i]	= (SCALAR *)calloc(1,MAXNP*sizeof(SCALAR));
      vy[i]	= (SCALAR *)calloc(1,MAXNP*sizeof(SCALAR));
      vz[i]	= (SCALAR *)calloc(1,MAXNP*sizeof(SCALAR));
      gamma_func[i]= (SCALAR *)calloc(1,MAXNP*sizeof(SCALAR));
    }

  px_hst= (SCALAR **)calloc(1,nsp*sizeof(SCALAR *));
  py_hst= (SCALAR **)calloc(1,nsp*sizeof(SCALAR *));
  pz_hst= (SCALAR **)calloc(1,nsp*sizeof(SCALAR *));

  for (i=0; i<nsp; i++)
    {
      px_hst[i]= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
      py_hst[i]= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
      pz_hst[i]= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
    }
  t_array= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
  bye_hst= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
  bze_hst= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
  exe_hst= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
  eye_hst= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
  eze_hst= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
  eyle_hst= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
  eyre_hst= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
  ezle_hst= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));
  ezre_hst= (SCALAR *)calloc(1,MAXHIST*sizeof(SCALAR));

  
  if (nsp && (!pos[nsp-1] || !vx[nsp-1] || !vy[nsp-1] || !vz[nsp-1]))
    {
      printf("START: pos, v malloc failed");
      exit(1);
    }

  /*********************************************************************/
  /******** Initializing Arrays ****************************************/
														/*Zero radiation fields at t=0*/
  for (i=0; i<ng; i++)
    {
      jym[i] = jyp[i] = jzm[i] = jzp[i] = 0.0;
      eyl[i] = eyr[i] = ezl[i] = ezr[i] = 0.0;
      thermx[i] = 0.0;
    }
  for (i=0; i<nsp; i++)
    for (j=0; j<MAXNP; j++)	
      pos[i][j] = vx[i][j] = vy[i][j] = vz[i][j] = 0.0;

  /******************************************************************/
	
  for (i=0; i<nsp; i++)	Creator(i,mode);
  if(InputDeck) fclose(InputDeck);

  /********************Initializing other variables******************/
  t	= 0.0;
  interval = 1;								/* Time history sample interval */
  hist_hi = 0;
  eloss = 0.0;
  rho0=0.0;
  for (i=0; i<nsp; i++)
    {
      rnmc2i[i] = (dt*dt)/(np[i]*ms[i]*dx*dx);
      rho0		-= np[i]*qs[i]/length;
    }
  tx[0] = wc1x*dt/2.0;
  tz[0] = wc1z*dt/2.0;
  for (i=1; i<nsp; i++)
    {
      tx[i]	= ms[0]*tx[0]/ms[i];
      tz[i] = ms[0]*tz[0]/ms[i];
    }
  for (i=0; i<nsp; i++)
    {
      px[i] = py[i] = pz[i] = 0.0;
    }
  /******************************************************************/
  
  for (i=0; i<nsp; i++)
    Setrho(i);
  if (dawsacc) Dawsacc(mno1,mno2,k0,forback,e01,e02,freq1,freq2,
		       rgtlft1,rgtlft2,wc1x,ey1,ey2,ez1,ez2);
  Fields();
  for (i=0; i<nsp; i++)
    Setv(i);
}
/***********************End of Start***********************************/

/************SETRHO converts position to computer normalization***********/
/******************and accumulates charge density at t=0.*****************/

Setrho(isp)
int isp;
{
  int i,j;
  SCALAR qdx, dxi, drho;

  qdx = qs[isp]/dx;
  dxi = 1.0/dx;
				/*If this is first group of particles, then clear out rho.*/
  if (isp == 0)
    {
      for (i=0; i<nc; i++)	rho[i] = rho0;
    }
  /*Linear weighting*/
  rho[nc] = 0.0;
  for (i=0; i<np[isp]; i++)
    {
      pos[isp][i] *= dxi;
      if (pos[isp][i] < 0) pos[isp][i] += nc;
      if (pos[isp][i] >= nc) pos[isp][i] -= nc;
      j	= pos[isp][i];
      drho	= qdx*(pos[isp][i]-j);
      rho[j]	+= qdx-drho;
      rho[j+1] += drho;
    }
  rho[0] += rho[nc];
  rho[nc] = rho[0];
}
/***************************End of SETRHO********************************/


/************* SETV converts particle velocities at t=0 to **************/
/*****************computer normalization at t=-dt/2. ********************/

Setv(isp)
int isp;
{
  int i,ij;
  SCALAR ae,abxpt,del,aexpt,aeypt,aezpt,abypt,abzpt,gvxs,gvys,
  gvzs,dcg,vvx,vvy,vvz,f;

  ae	= 0.5*(qs[isp]/ms[isp])*dt*dt/dx;
  abxpt = -0.5*tx[isp];
  for (i=0; i<np[isp]; i++)
    {
      /* Interpolate fields to particle position */
      ij	= pos[isp][i];
      del= pos[isp][i]-ij;
      gamma_func[isp][i]= 1.0/sqrt(1.0-vx[isp][i]*vx[isp][i] -
				   vy[isp][i]*vy[isp][i] - vz[isp][i]*vz[isp][i]);
      aexpt = -0.5*ae*(ex[ij] + del*(ex[ij+1]-ex[ij]));
      aeypt = -0.5*ae*(ey[ij] + del*(ey[ij+1]-ey[ij]));
      aezpt = -0.5*ae*(ez[ij] + del*(ez[ij+1]-ez[ij]));
      abypt = -0.5*ae*(by[ij] + del*(by[ij+1]-by[ij]));
      abzpt = -0.5*ae*(bz[ij] + del*(bz[ij+1]-bz[ij]) + tz[isp]);

      /* Half-acceleration along E. */
      gvxs	= vx[isp][i]*gamma_func[isp][i];
      gvys	= vy[isp][i]*gamma_func[isp][i];
      gvzs	= vz[isp][i]*gamma_func[isp][i];

      /*First half-rotation, using v x B /gamma_func.*/
      gamma_func[isp][i]= sqrt(1.0+gvxs*gvxs + gvys*gvys + gvzs*gvzs);
      dcg	= 1.0/(dx*gamma_func[isp][i]/dt);
      abxpt	*= dcg;
      abypt *= dcg;
      abzpt *= dcg;
      vvx	= gvxs + gvys*abzpt - gvzs*abypt;
      vvy	= gvys + gvzs*abxpt - gvxs*abzpt;
      vvz	= gvzs + gvxs*abypt - gvys*abxpt;
      
      /*Second half-rotation and second half-acceleration*/
      f		= 2.0/(1.0+abxpt*abxpt+abypt*abypt+abzpt*abzpt);
      abxpt	*= f;
      abypt *= f;
      abzpt *= f;
      gvxs	+= vvy*abzpt - vvz*abypt + aexpt;
      gvys	+= vvz*abxpt - vvx*abzpt + aeypt;
      gvzs	+= vvx*abypt - vvy*abxpt + aezpt;

      /* Use v=p/(mc*gamma_func) */
      gamma_func[isp][i] = sqrt(1.0+gvxs*gvxs + gvys*gvys + gvzs*gvzs);
      vx[isp][i] = gvxs/gamma_func[isp][i];
      vy[isp][i] = gvys/gamma_func[isp][i];
      vz[isp][i] = gvzs/gamma_func[isp][i];
      px[isp] 	+= gvxs/np[isp];
      py[isp]	+= gvys/np[isp];
      pz[isp]	+= gvzs/np[isp];
    }
}
