/**************************************************************************
	FIELDS solves for phi and ex, computes field energy,
	and advances the wave radiation fields.
**************************************************************************/

#include "em1.h"

Fields()
{
  static int nc2=0;
  static SCALAR hdx,hdxi,pi,lengthi,tem;
  SCALAR kdx2,temp,eses,eyles,eyres,eyes,bzes,ezles,ezres,ezes,
        byes;

  int i, k, ii;

  if (!nc2)
    {
      nc2 = nc/2;
      hdx = 0.5*dx;
      hdxi= 0.5/dx;
      lengthi = 1.0/length;
      tem = .25*cgshl*dt;
      /* Set up ratio phi/rho */
      pi = TWOPI/2.0;
      for (i=0; i<nc2; i++)
	{
	  kdx2 = pi*(i+1)/ng;
	  temp = 2.0*sin(kdx2)/dx;
	  ksq[i] = cgshl/(temp*temp);
	}
    }
  
  /* Transform charge density */
  for (i=0; i<nc; i++)
    {
      rho[i] *= hdx;
      ex[i] = 0.0;
    }
  
  /* Add one to index for fft's */
  cpft (rho-1,ex-1, nc, 1, 1.0);
  rpft2(rho-1,ex-1,nc,1);
  
  /* Subtract one from index */
  rho[0] = eses = phi[0] = 0.0;
  
  /* Calculate phik and field energy */
  /* Compute the energy in a particular */
  /* fourier mode. */
  /* Fourier energy time history not finished */
  /********************************************/
  
  for (i=1; i<nc2; i++)
    {
      ii=nc-i;
      phi[i] = ksq[i-1]*rho[i];
      phi[ii]= ksq[i-1]*rho[ii];
      ek[i]	 = rho[i]*phi[i]+rho[ii]*phi[ii];
      eses 	+= ek[i];
      ek[i] *= rnmc2i[0]/length;
    }
  phi[nc2] = ksq[nc2-1]*rho[nc2];
  exe = (2.0*eses+rho[nc2]*phi[nc2])/(2.0*length)*rnmc2i[0];
  ek[nc2] = rho[nc2]*phi[nc2]*rnmc2i[0]/(2.0*length);
  ek[0] = 0.0;

  /* Inverse transform phi */
  for (i=0; i<nc; i++)
    {
      rho[i] *= lengthi;
      phi[i] *= lengthi;
    }
  
  /* Add one to index for fft's */
  rpfti2(phi-1,rho-1,nc,1);
  cpft(phi-1,rho-1,nc,1,-1.0);
  
  /* Subtract one from index */
  phi[nc] = phi[0];
  rho[nc] = rho[0];
  
  /* Centered difference across 2 cells */
  for (i=1; i<nc; i++)
      ex[i] = (phi[i-1]-phi[i+1])*hdxi;

  ex[0] = (phi[nc-1]-phi[1])*hdxi;
  ex[nc]= ex[0];
  
  /* Maxwell equation solver for */
  /* radiation fields */
  /* Add half of current, formed from */
  /* pos[t] and vy[t+dt/2]. */
  for (i=0; i<ng; i++)
    {
      eyl[i] -= tem*jym[i];
      eyr[i] -= tem*jym[i];
      ezl[i] -= tem*jzm[i];
      ezr[i] -= tem*jzm[i];
    }
  
  /* Let c=dx/dt. Then eyl,ezl are just moved one*/
  /* cell to left and eyr,ezr one cell to right.*/
  for (i=0; i<nc; i++)
    {
      eyl[i] = eyl[i+1];
      ezl[i] = ezl[i+1];
      ii = nc-i;
      eyr[ii] = eyr[ii-1];
      ezr[ii] = ezr[ii-1];
    }
  eyl[nc]	= eyl[0];
  eyr[0]	= eyr[nc];
  ezl[nc]	= ezl[0];
  ezr[0]	= ezr[nc];
  
  /* Add other half of current, formed from */
  /* pos[d+dt] and vy[t+dt/2]. */
  for (i=0; i<ng; i++)
    {
      eyl[i] -= tem*jyp[i];
      eyr[i] -= tem*jyp[i];
      ezl[i] -= tem*jzp[i];
      ezr[i] -= tem*jzp[i];
    }
  for (i=0; i<ng; i++)
    {
      ey[i] = eyr[i] + eyl[i];
      bz[i] = eyr[i] - eyl[i];
      ez[i] = ezr[i] + ezl[i];
      by[i] = ezl[i] - ezr[i];
    }
  
  /* Calculate radiation field energies */
  eyles = eyres = eyes = bzes = ezles = ezres = ezes = byes = 0.0;

  for (i=1; i<nc; i++)
    {
      eyles += eyl[i]*eyl[i];
      eyres += eyr[i]*eyr[i];
      bzes	+= bz[i]*bz[i];
      eyes	+= ey[i]*ey[i];
      ezles	+= ezl[i]*ezl[i];
      byes	+= by[i]*by[i];
      ezes	+= ez[i]*ez[i];
    }
  eyles	+= 0.5*(eyl[0]*eyl[0] + eyl[nc]*eyl[nc]);
  eyres += 0.5*(eyr[0]*eyr[0] + eyr[nc]*eyr[nc]);
  eyes	+= 0.5*(ey[0]*ey[0] + ey[nc]*ey[nc]);
  bzes	+= 0.5*(bz[0]*bz[0] + bz[nc]*bz[nc]);
  ezles	+= 0.5*(ezl[0]*ezl[0] + ezl[nc]*ezl[nc]);
  ezres += 0.5*(ezr[0]*ezr[0] + ezr[nc]*ezr[nc]);
  ezes	+= 0.5*(ez[0]*ez[0] + ez[nc]*ez[nc]);
  byes	+= 0.5*(by[0]*by[0] + by[nc]*by[nc]);
  eyle	= eyles*dx/cgshl*rnmc2i[0];
  eyre	= eyres*dx/cgshl*rnmc2i[0];
  eye	= eyes*dx/(2.0*cgshl)*rnmc2i[0];
  bze	= bzes*dx/(2.0*cgshl)*rnmc2i[0];
  ezle	= ezles*dx/cgshl*rnmc2i[0];
  ezre	= ezres*dx/cgshl*rnmc2i[0];
  eze	= ezes*dx/(2.0*cgshl)*rnmc2i[0];
  bye	= byes*dx/(2.0*cgshl)*rnmc2i[0];
  
  /* Energy histories not computed */
  /*********************************/
}

